' This file is part of Network Switcher.

' Network Switcher is free software: you can redistribute it and/or modify
' it under the terms of the GNU General Public License as published by
' the Free Software Foundation, either version 3 of the License, or
' (at your option) any later version.

' Network Switcher is distributed in the hope that it will be useful,
' but WITHOUT ANY WARRANTY; without even the implied warranty of
' MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
' GNU General Public License for more details.

' You should have received a copy of the GNU General Public License
' along with Network Switcher. If not, see <http://www.gnu.org/licenses/>.
'
' Copyright 2007, Ricardo Amaral

Imports System.Reflection
Imports NetworkSwitcher.Interfaces

Module ModuleMain

#Region " Global Application Declarations "

    Friend Structure StructAppSettings
        Dim WindowsStartup As Boolean
        Dim StartHidden As Boolean
        Dim ExitOnClosing As Boolean
        Dim NicFolder As String
        Dim EnableVerb As String
        Dim DisableVerb As String
        Dim DirectConnection As Boolean
        Dim ProxyAddress As String
        Dim ProxyPort As Integer
        Dim BypassLocal As Boolean
    End Structure

    Friend Structure StructConnSettings
        Dim Enable As Boolean
        Dim Name As String
        Dim DHCP As Boolean
        Dim IP As String
        Dim Subnet As String
        Dim Gateway As String
        Dim DNS1 As String
        Dim DNS2 As String
    End Structure

    Friend Enum ToggleMode
        Toggle
        Enable
        Disable
    End Enum

    Friend Structure StructAvailablePlugins
        Dim Name As String
        Dim Methods As ArrayList
        Dim Checked As ArrayList
    End Structure

    Friend Structure StructShellItems
        Dim NicFolder As Shell32.Folder
        Dim NicItems() As String
    End Structure

    Dim frmMain As FormProfilesManager

    Friend Settings As StructAppSettings
    Friend ShellItems As StructShellItems
    Friend newProfile As Boolean
    Friend editProfile As Boolean
    Friend profileName As String
    Friend realVersion As String
    Friend XML As XMLParser
    Friend saveProfile As Boolean = True
    Friend listProfiles As New ArrayList
    Friend AvailablePlugins() As PluginServices.AvailablePlugin
    Friend objHost As IHost
    Friend dicMethods As New Dictionary(Of String, PluginDelegate)
    Friend Plugins As New ArrayList

#End Region

#Region " Sub Main "

    Public Sub Main()
        Dim aModuleName As String = Process.GetCurrentProcess.MainModule.ModuleName
        Dim aProcName As String = IO.Path.GetFileNameWithoutExtension(aModuleName)

        ' Enable Windows Visual Styles
        Application.EnableVisualStyles()

        ' Do not allow more than one instance of our application
        If Process.GetProcessesByName(aProcName).Length > 1 Then
            ' Output error message and exit application
            MessageBox.Show("Network Switcher is already running." + ControlChars.CrLf + "Only one instance of the application is allowed to be running.", "Application already running!", MessageBoxButtons.OK, MessageBoxIcon.Information)
            Application.Exit()
        Else
            ' Create a new form instance of Profiles Manager
            frmMain = New FormProfilesManager

            ' Set the application title in various places of frmMain
            realVersion = Application.ProductVersion
            frmMain.Text = Application.ProductName + " v" + realVersion

            ' Create and initialize a new host interface
            objHost = New PluginsHost()

            ' Load settings, profiles and plugins
            LoadSettings()
            LoadProfiles()
            LoadPlugins()

            ' Load network connections folder
            If Settings.NicFolder IsNot Nothing Then
                ShellItems.NicFolder = GetNetworkConnectionsFolder(Settings.NicFolder)
            End If

            ' Load all network connection cards available
            ShellItems.NicItems = GetAllNetworkAdapters()

            ' Sets proxy settings to plugins interface
            SetProxySettingsToInterface()

            ' Starts running the application with main window hidden?
            If Settings.StartHidden Then : Application.Run()
            Else : Application.Run(frmMain)
            End If
        End If
    End Sub

#End Region

#Region " Private Methods "

    ' Method: Load application settings
    Private Sub LoadSettings()
        ' Initialize XML settings file
        XML = New XMLParser(Application.StartupPath, "settings", "Settings")

        ' Load all settings from xml
        Settings.WindowsStartup = Boolean.Parse(XML.GetSettingStr("Global", "WindowsStartup", False))
        Settings.StartHidden = Boolean.Parse(XML.GetSettingStr("Global", "StartHidden", False))
        Settings.ExitOnClosing = Boolean.Parse(XML.GetSettingStr("Global", "ExitOnClosing", False))
        Settings.NicFolder = XML.GetSettingStr("Global", "NICFolder", Nothing)
        Settings.EnableVerb = XML.GetSettingStr("Global", "EnableVerb", Nothing)
        Settings.DisableVerb = XML.GetSettingStr("Global", "DisableVerb", Nothing)
        Settings.DirectConnection = Boolean.Parse(XML.GetSettingStr("Proxy", "DirectConnection", True))
        Settings.ProxyAddress = XML.GetSettingStr("Proxy", "Address", Nothing)
        Settings.ProxyPort = XML.GetSettingInt("Proxy", "Port", Nothing)
        Settings.BypassLocal = Boolean.Parse(XML.GetSettingStr("Proxy", "BypassLocal", False))
    End Sub

    ' Method: Load application profiles
    Private Sub LoadProfiles()
        Dim profiles() As String

        ' Initialize XML profiles file
        XML = New XMLParser(Application.StartupPath, "profiles", "Profiles")

        ' Get all profile names
        profiles = XML.GetSectionNames()

        ' Create array with all the profiles
        For Each profile As String In profiles
            listProfiles.Add(profile)
        Next

        ' Sort the array alphabetically
        listProfiles.Sort()

        ' Add profiles to combobox on main form and notify area
        RefreshProfiles(Nothing)
    End Sub

    ' Method: Finds and loads all plugins and their methods
    Private Sub LoadPlugins()
        Dim objPlugin As IPlugin
        Dim pmItem As StructAvailablePlugins
        Dim chkItems As ArrayList

        ' Finds all plugins listed in the application directory
        AvailablePlugins = PluginServices.FindPlugins(Application.StartupPath + "\plugins", _
        "NetworkSwitcher.Interfaces.IPlugin")

        ' Loops through all available plugins and get their names and methods
        If Not AvailablePlugins Is Nothing Then
            For Each plugin As PluginServices.AvailablePlugin In AvailablePlugins
                ' Create plugin instance and initialize it
                objPlugin = DirectCast(PluginServices.CreateInstance(plugin), IPlugin)
                objPlugin.Initialize(objHost)

                ' Set plugin name and methods to pluginMethods Structure
                pmItem.Name = objPlugin.PluginName
                pmItem.Methods = GetPluginMethods(objPlugin)

                ' Set all plugin methods to unchecked
                chkItems = New ArrayList

                For I As Integer = 0 To pmItem.Methods.Count - 1
                    chkItems.Add(CheckState.Unchecked)
                Next

                pmItem.Checked = chkItems

                ' Add this pluginMethods Structure to the Plugins ArrayList
                Plugins.Add(pmItem)
            Next
        End If
    End Sub

    ' Method: Get all plugin methods and add them to ArrayList and Dictionary
    Private Function GetPluginMethods(ByVal plugin As IPlugin) As ArrayList
        Dim pMethods As PluginDelegate()
        Dim pType As Type
        Dim pmItems As New ArrayList

        ' Get the plugin type
        pType = DirectCast(plugin, Object).GetType()

        ' Get the plugin methods
        pMethods = plugin.GetMethods()

        For Each pd As PluginDelegate In pMethods
            ' Get the type.method name and add it to a Dictionary
            dicMethods.Add(pType.Name + "." + pd.Method.Name, pd)

            ' Add the method to the methods ArrayList
            pmItems.Add(pd.Method.Name)
        Next

        Return pmItems
    End Function

    ' Method: Parse ips and check if they are "empty" (0.0.0.0)
    Private Function ParseIP4Label(ByVal ip As String) As String
        If ip = "0.0.0.0" Or ip = Nothing Then
            Return Nothing
        Else
            Return ip.Replace(".", " . ")
        End If
    End Function

    ' Method: Gets all plugins assigned to a profile
    Private Function GetAssignedPlugins(ByVal profile As String) As String()
        Dim pCount As Integer = XML.GetSettingInt(profile, "PluginsCount", 0)
        Dim aPlugins(pCount - 1) As String

        ' Loops through all plugins and return the ones assigned to the profile
        For I As Integer = 0 To pCount - 1
            aPlugins(I) = XML.GetSettingStr(profile, "PluginName(" + CStr(I) + ")", Nothing)
        Next

        Return aPlugins
    End Function

    ' Invokes a plugin method
    Private Sub CallPluginMethod(ByVal typeMethod As String)
        Dim PluginMethod As PluginDelegate

        ' Find the method in the Dictionary
        PluginMethod = dicMethods(typeMethod)

        ' Call the method if it was found
        If (PluginMethod IsNot Nothing) Then
            PluginMethod()
        End If
    End Sub

#End Region

#Region " Friend Methods "

    ' Method: Populate profiles combobox and notify menu (on main form)
    Friend Sub RefreshProfiles(ByVal selectedProfile As String)
        Dim i As Integer, mnuItem As ToolStripMenuItem

        ' Clear combobox
        frmMain.cbxProfiles.Items.Clear()

        ' Add profiles to combobox
        For i = 0 To listProfiles.Count - 1
            frmMain.cbxProfiles.Items.Add(listProfiles(i))
        Next

        ' Do we have a profile to select by default?
        If Not selectedProfile Is Nothing Then
            frmMain.cbxProfiles.SelectedItem = selectedProfile
        End If

        ' Clear notifymenu
        frmMain.notifyMenu.Items.Clear()

        ' Add notifymenu items before the profiles
        frmMain.notifyMenu.Items.AddRange(New System.Windows.Forms.ToolStripItem() _
            {frmMain.mnuHeader, frmMain.mnuManageProfiles, frmMain.mnuSettings, frmMain.mnuSeparator1})

        ' Disable the notify menu show item?
        If Not Settings.StartHidden Then
            frmMain.notifyMenu.Items(1).Enabled = False
        End If

        ' Add profiles to notifymenu
        If listProfiles.Count = 0 Then
            frmMain.notifyMenu.Items.Add(New ToolStripMenuItem("<<No profiles>>"))
            frmMain.notifyMenu.Items(4).Enabled = False

        Else
            For i = 0 To listProfiles.Count - 1
                mnuItem = New ToolStripMenuItem(listProfiles(i), Nothing, New System.EventHandler(AddressOf frmMain.mnuItem_Click))
                frmMain.notifyMenu.Items.Add(mnuItem)
            Next
        End If

        ' Add notifymenu items after the profiles
        frmMain.notifyMenu.Items.AddRange(New System.Windows.Forms.ToolStripItem() _
            {frmMain.mnuSeparator2, frmMain.mnuHelp, frmMain.mnuAbout, frmMain.mnuSeparator3, frmMain.mnuExit})
    End Sub

    ' Method: Gets connection settings from XML into a structure
    Friend Function GetStructConnSettings(ByVal XML As XMLParser, ByVal profile As String, ByVal key As String) As StructConnSettings
        Dim Settings As StructConnSettings

        ' Get this connection settings
        Settings.Enable = Boolean.Parse(XML.GetSettingStr(profile, key + "Enable", False))
        Settings.Name = XML.GetSettingStr(profile, key + "Name", Nothing)
        Settings.DHCP = Boolean.Parse(XML.GetSettingStr(profile, key + "DHCP", False))
        Settings.IP = XML.GetSettingStr(profile, key + "IP", Nothing)
        Settings.Subnet = XML.GetSettingStr(profile, key + "Subnet", Nothing)
        Settings.Gateway = XML.GetSettingStr(profile, key + "Gateway", Nothing)
        Settings.DNS1 = XML.GetSettingStr(profile, key + "DNS1", Nothing)
        Settings.DNS2 = XML.GetSettingStr(profile, key + "DNS2", Nothing)

        ' Returns this connection settings
        Return Settings
    End Function

    ' Method: Applys the new network profile to the system
    Friend Sub ApplyProfile(ByVal profile As String)
        Dim NIC1 As StructConnSettings, NIC2 As StructConnSettings
        Dim assignedPlugins() As String

        ' Initialize XML profiles file
        XML = New XMLParser(Application.StartupPath, "profiles", "Profiles")

        ' Gets NIC1 and NIC2 profile settings
        NIC1 = GetStructConnSettings(XML, profile, "NIC1")
        NIC2 = GetStructConnSettings(XML, profile, "NIC2")

        ' Get assigned plugins names to this profile
        assignedPlugins = GetAssignedPlugins(profile)

        ' Process first network connection?
        If NIC1.Name IsNot Nothing Then
            ' Is this connection going to be enabled or disabled?
            If NIC1.Enable Then
                ' Enable connection card
                ToggleNic(NIC1.Name, ToggleMode.Enable)

                ' Waits for the connection card to be enabled
                'While Not IsNICActive(NIC1.Name)
                '    System.Threading.Thread.Sleep(1000)
                'End While

                ' Enable or disable DHCP?
                If NIC1.DHCP Then
                    Dim errorDHCP As Boolean = WMIEnableDHCP(NIC1.Name)

                    ' Enable DHCP with the Registry if WMI failed
                    If errorDHCP Then
                        RegistryEnableDHCP(NIC1.Name)
                    End If
                Else
                    ' Enable static IPs
                    WMIEnableStatic(NIC1.Name, NIC1.IP, NIC1.Subnet, NIC1.Gateway, _
                    NIC1.DNS1 + "," + NIC1.DNS2)
                End If
            Else
                ' Disable connection card
                ToggleNic(NIC1.Name, ToggleMode.Disable)
            End If
        End If

        ' Process second network connection?
        If NIC2.Name IsNot Nothing Then
            ' Is this connection going to be enabled or disabled?
            If NIC2.Enable Then
                ' Enable connection card
                ToggleNic(NIC2.Name, ToggleMode.Enable)

                ' Waits for the connection card to be enabled
                'While Not IsNICActive(NIC2.Name)
                '    System.Threading.Thread.Sleep(1000)
                'End While

                ' Enable or disable DHCP?
                If NIC2.DHCP Then
                    Dim errorDHCP As Boolean = WMIEnableDHCP(NIC2.Name)

                    ' Enable DHCP with the Registry if WMI failed
                    If Not errorDHCP Then
                        RegistryEnableDHCP(NIC2.Name)
                    End If
                Else
                    ' Enable static IPs
                    WMIEnableStatic(NIC2.Name, NIC2.IP, NIC2.Subnet, NIC2.Gateway, _
                    NIC2.DNS1 + "," + NIC2.DNS2)
                End If
            Else
                ' Disable connection card
                ToggleNic(NIC2.Name, ToggleMode.Disable)
            End If
        End If

        ' Do we have any assigned plugins to process?
        If assignedPlugins.Length > 0 And AvailablePlugins IsNot Nothing Then
            Dim pCount As Integer = XML.GetSettingInt(profile, "PluginsCount", 0)
            Dim pMethods As String, pClassName As String, pName As String
            Dim objPlugin As IPlugin

            ' Loops through all avaialble plugins
            For Each plugin As PluginServices.AvailablePlugin In AvailablePlugins
                ' Trims the class name from the full class name
                pClassName = plugin.ClassName.Remove(0, plugin.ClassName.IndexOf(".") + 1)

                ' Create this plugin instance
                objPlugin = DirectCast(PluginServices.CreateInstance(plugin), IPlugin)

                ' Loops through all plugins assigned to this profile
                For I As Integer = 0 To pCount - 1
                    pName = XML.GetSettingStr(profile, "PluginName(" + CStr(I) + ")", Nothing)

                    ' Is this the plugin we want to run?
                    If pName = objPlugin.PluginName Then
                        ' Gets all methods to execute for this plugin from the XML
                        pMethods = XML.GetSettingStr(profile, "PluginMethods(" + CStr(I) + ")", Nothing)

                        ' Call all the assigned methods from this plugin
                        For Each pMethod As String In pMethods.Split(";")
                            CallPluginMethod(pClassName + "." + pMethod)
                        Next
                    End If
                Next
            Next
        End If

        ' Parse IPs
        NIC1.IP = ParseIP4Label(NIC1.IP)
        NIC1.Subnet = ParseIP4Label(NIC1.Subnet)
        NIC1.Gateway = ParseIP4Label(NIC1.Gateway)
        NIC1.DNS1 = ParseIP4Label(NIC1.DNS1)
        NIC1.DNS2 = ParseIP4Label(NIC1.DNS2)
        NIC2.IP = ParseIP4Label(NIC2.IP)
        NIC2.Subnet = ParseIP4Label(NIC2.Subnet)
        NIC2.Gateway = ParseIP4Label(NIC2.Gateway)
        NIC2.DNS1 = ParseIP4Label(NIC2.DNS1)
        NIC2.DNS2 = ParseIP4Label(NIC2.DNS2)

        ' Clear plugins list and plugin about info
        frmMain.lstAssignedPlugins.Items.Clear()
        frmMain.lblPluginAuthor.Text = Nothing
        frmMain.lblPluginVersion.Text = Nothing
        frmMain.lblPluginDescription.Text = Nothing

        ' Populate form with profile settings
        If Not NIC1.Name = Nothing Then
            If NIC1.Enable Then
                frmMain.picEnabledN1.Image = My.Resources.Label_YES
            Else
                frmMain.picEnabledN1.Image = My.Resources.Label_NO
            End If
        Else
            frmMain.picEnabledN1.Image = My.Resources.Label_UNCONFIGURED
        End If

        If NIC1.DHCP Then
            frmMain.picDHCPN1.Image = My.Resources.Label_DHCP
            frmMain.elblIPN1.Text = Nothing
            frmMain.elblSubnetN1.Text = Nothing
            frmMain.elblGatewayN1.Text = Nothing
            frmMain.elblDNS1N1.Text = Nothing
            frmMain.elblDNS2N1.Text = Nothing
        Else
            frmMain.picDHCPN1.Image = Nothing
            frmMain.elblIPN1.Text = NIC1.IP
            frmMain.elblSubnetN1.Text = NIC1.Subnet
            frmMain.elblGatewayN1.Text = NIC1.Gateway
            frmMain.elblDNS1N1.Text = NIC1.DNS1
            frmMain.elblDNS2N1.Text = NIC1.DNS2
        End If

        ' Populate form with profile settings
        If Not NIC2.Name = Nothing Then
            If NIC2.Enable Then
                frmMain.picEnabledN2.Image = My.Resources.Label_YES
            Else
                frmMain.picEnabledN2.Image = My.Resources.Label_NO
            End If
        Else
            frmMain.picEnabledN2.Image = My.Resources.Label_UNCONFIGURED
        End If

        If NIC2.DHCP Then
            frmMain.picDHCPN2.Image = My.Resources.Label_DHCP
            frmMain.elblIPN2.Text = Nothing
            frmMain.elblSubnetN2.Text = Nothing
            frmMain.elblGatewayN2.Text = Nothing
            frmMain.elblDNS1N2.Text = Nothing
            frmMain.elblDNS2N2.Text = Nothing
        Else
            frmMain.picDHCPN2.Image = Nothing
            frmMain.elblIPN2.Text = NIC2.IP
            frmMain.elblSubnetN2.Text = NIC2.Subnet
            frmMain.elblGatewayN2.Text = NIC2.Gateway
            frmMain.elblDNS1N2.Text = NIC2.DNS1
            frmMain.elblDNS2N2.Text = NIC2.DNS2
        End If

        ' Do we have any plugins to display in the main window?
        If assignedPlugins.Length > 0 Then
            ' Loops through all assigned plugins
            For Each plugin As String In assignedPlugins
                ' Lopps through all avaialble plugins
                For Each pmItem As StructAvailablePlugins In Plugins
                    ' Adds the assigned plugin to main form?
                    If plugin = pmItem.Name Then
                        frmMain.lstAssignedPlugins.Items.Add(plugin)

                        Exit For
                    End If
                Next

            Next
        End If

        ' Enable all groupboxes inside the last activated profile panel
        frmMain.gbxNIC1.Enabled = True
        frmMain.gbxNIC2.Enabled = True
        frmMain.gbxAssignedPlugins.Enabled = True
        frmMain.gbxAboutPlugin.Enabled = True

        ' Set this active profile in the combobox
        For I As Integer = 0 To frmMain.cbxProfiles.Items.Count - 1
            If frmMain.cbxProfiles.Items.Item(I) = profile Then
                frmMain.cbxProfiles.SelectedIndex = I
            End If
        Next
    End Sub

    ' Method: Sets proxy settings on Host Interface so plugins can use them
    Friend Sub SetProxySettingsToInterface()
        objHost.DirectConnection = Settings.DirectConnection
        objHost.ProxyAddress = Settings.ProxyAddress
        objHost.ProxyPort = Settings.ProxyPort
        objHost.BypassLocal = Settings.BypassLocal
    End Sub

#End Region

End Module
